@@ -30,9 +30,9 @@ CQL::Lexer since the lexer is used behind the scenes by CQL::Parser.
 =cut
 
 ## CQL keyword types
-use constant CQL_LT        => 100;      ## The "<" relation
-use constant CQL_GT        => 101;      ## The ">" relation
-use constant CQL_EQ        => 102;      ## The "=" relation
+use constant CQL_LT        => 100;  ## The "<" relation
+use constant CQL_GT        => 101;  ## The ">" relation
+use constant CQL_EQ        => 102;  ## The "=" relation
 use constant CQL_LE        => 103;	## The "<=" relation
 use constant CQL_GE        => 104;	## The ">=" relation
 use constant CQL_NE        => 105;	## The "<>" relation
@@ -43,7 +43,7 @@ use constant CQL_PROX      => 109;	## The "prox" boolean
 use constant CQL_ANY       => 110;	## The "any" relation
 use constant CQL_ALL       => 111;	## The "all" relation
 use constant CQL_EXACT     => 112;	## The "exact" relation
-use constant CQL_PWORD     => 113;	## The "word" proximity unit
+use constant CQL_PWORD     => 113;	## The "word" proximity unit and the "word" relation modifier
 use constant CQL_SENTENCE  => 114;	## The "sentence" proximity unit
 use constant CQL_PARAGRAPH => 115;	## The "paragraph" proximity unit
 use constant CQL_ELEMENT   => 116;	## The "element" proximity unit
@@ -54,11 +54,17 @@ use constant CQL_FUZZY     => 120;	## The "fuzzy" relation modifier
 use constant CQL_STEM      => 121;	## The "stem" relation modifier
 use constant CQL_SCR       => 122;	## The server choice relation
 use constant CQL_PHONETIC  => 123;	## The "phonetic" relation modifier
-use constant CQL_WORD      => 124;      ## A general word (not an operator) 
-use constant CQL_LPAREN    => 125;      ## A left paren
-use constant CQL_RPAREN    => 126;      ## A right paren
-use constant CQL_EOF       => 127;      ## End of query
-use constant CQL_MODIFIER  => 128;      ## Start of modifier '/'
+use constant CQL_WORD      => 124;  ## A general word (not an operator) 
+use constant CQL_LPAREN    => 125;  ## A left paren
+use constant CQL_RPAREN    => 126;  ## A right paren
+use constant CQL_EOF       => 127;  ## End of query
+use constant CQL_MODIFIER  => 128;  ## Start of modifier '/'
+use constant CQL_STRING    => 129;  ## The "string" relation modifier
+use constant CQL_ISODATE   => 130;  ## The "isoDate" relation modifier
+use constant CQL_NUMBER    => 131;  ## The "number" relation modifier
+use constant CQL_URI       => 132;  ## The "uri" relation modifier
+use constant CQL_MASKED    => 133;  ## The "masked" relation modifier
+use constant CQL_UNMASKED  => 134;  ## The "unmasked" relation modifier
 
 ## lookup table for easily determining token type
 our %lookupTable = (
@@ -89,6 +95,12 @@ our %lookupTable = (
     ')'          => CQL_RPAREN,
     '/'          => CQL_MODIFIER,
     ''           => CQL_EOF,
+    'string'     => CQL_STRING,
+    'isodate'    => CQL_ISODATE,
+    'number'     => CQL_NUMBER,
+    'uri'        => CQL_URI,
+    'masked'     => CQL_MASKED,
+    'unmasked'   => CQL_UNMASKED
 );
 
 ## constants available for folks to use when looking at 
@@ -99,7 +111,8 @@ our @EXPORT = qw(
     CQL_PROX CQL_ANY CQL_ALL CQL_EXACT CQL_PWORD CQL_SENTENCE CQL_PARAGRAPH
     CQL_ELEMENT CQL_ORDERED CQL_UNORDERED CQL_RELEVANT CQL_FUZZY
     CQL_STEM CQL_SCR CQL_PHONETIC CQL_RPAREN CQL_LPAREN
-    CQL_WORD CQL_PHRASE CQL_EOF CQL_MODIFIER
+    CQL_WORD CQL_PHRASE CQL_EOF CQL_MODIFIER CQL_STRING CQL_ISODATE
+    CQL_NUMBER CQL_URI CQL_MASKED CQL_UNMASKED
 );
 
 =head2 new()
@@ -113,12 +126,17 @@ sub new {
     # see if it's a reserved word, which are case insensitive
     my $normalString = lc($string);
     if ( exists($lookupTable{$normalString}) ) {
-        $string = $normalString;
         $type = $lookupTable{$normalString};
     }
     else {
         $type = CQL_WORD;
-        $string =~ s/"//g; # remove quotes if present
+        # remove outer quotes if present
+        if ($string =~ m/^"(.*)"$/g) {
+        	$string = $1;
+        	# replace escaped double quote with double quote. 
+        	# Is save this way cause the string is assumed to be syntactically correct
+        	$string =~ s/\\"/"/g;
+        }
     }
     return bless { string=>$string, type=>$type }, ref($class) || $class;
 }
